@@ -13,7 +13,54 @@
 #include <pthread.h>
 #endif
 
+//#define LFZ_DEBUG_MUTEXES
+#ifdef LFZ_DEBUG_MUTEXES
+
+#include <list>
+#include <memory>
+#include <thread>
+
 namespace fz {
+class mutex;
+class mutex_debug;
+
+/// \private
+struct FZ_PUBLIC_SYMBOL lock_order final {
+	std::vector<mutex*> mutexes_;
+	std::vector<void*> backtrace_;
+};
+
+/// \private
+struct FZ_PUBLIC_SYMBOL mutex_debug final
+{
+	static void record_lock(void* m);
+	static void record_unlock(void* m);
+
+	size_t count_{};
+	std::thread::id id_{};
+	std::vector<std::list<lock_order>::iterator> own_orders_;
+	bool unchecked_{};
+};
+}
+#endif
+
+namespace fz {
+enum class mutex_flags
+{
+	recursive,
+
+#ifdef LFZ_DEBUG_MUTEXES
+	/// \private
+	debug_unchecked,
+#endif
+};
+inline bool operator&(mutex_flags lhs, mutex_flags rhs) {
+	return (static_cast<std::underlying_type_t<mutex_flags>>(lhs) & static_cast<std::underlying_type_t<mutex_flags>>(rhs)) != 0;
+}
+inline mutex_flags operator|(mutex_flags lhs, mutex_flags rhs)
+{
+	return static_cast<mutex_flags>(static_cast<std::underlying_type_t<mutex_flags>>(lhs) | static_cast<std::underlying_type_t<mutex_flags>>(rhs));
+}
 
 /**
  * \brief Lean replacement for std::(recursive_)mutex
@@ -28,6 +75,7 @@ class FZ_PUBLIC_SYMBOL mutex final
 {
 public:
 	explicit mutex(bool recursive = true);
+	explicit mutex(mutex_flags flags);
 	~mutex();
 
 	mutex(mutex const&) = delete;
@@ -51,6 +99,10 @@ private:
 #else
 	pthread_mutex_t m_;
 #endif
+#ifdef LFZ_DEBUG_MUTEXES
+public:
+	mutex_debug debug_;
+#endif
 };
 
 /** \brief A simple scoped lock.
@@ -64,6 +116,11 @@ private:
 class FZ_PUBLIC_SYMBOL scoped_lock final
 {
 public:
+	enum flag
+	{
+		try_lock
+	};
+
 	explicit scoped_lock(mutex& m)
 		: m_(&m.m_)
 	{
@@ -72,11 +129,25 @@ public:
 #else
 		pthread_mutex_lock(m_);
 #endif
+#ifdef LFZ_DEBUG_MUTEXES
+		mutex_debug::record_lock(m_);
+#endif
+	}
+
+	explicit scoped_lock(mutex& m, flag)
+	    : m_(&m.m_)
+	{
+		if (!m.try_lock()) {
+			locked_ = false;
+		}
 	}
 
 	~scoped_lock()
 	{
 		if (locked_) {
+#ifdef LFZ_DEBUG_MUTEXES
+			mutex_debug::record_unlock(m_);
+#endif
 #ifdef FZ_WINDOWS
 			LeaveCriticalSection(m_);
 #else
@@ -119,6 +190,9 @@ public:
 		EnterCriticalSection(m_);
 #else
 		pthread_mutex_lock(m_);
+#endif
+#ifdef LFZ_DEBUG_MUTEXES
+		mutex_debug::record_lock(m_);
 #endif
 	}
 
@@ -129,6 +203,9 @@ public:
 	void unlock()
 	{
 		locked_ = false;
+#ifdef LFZ_DEBUG_MUTEXES
+		mutex_debug::record_unlock(m_);
+#endif
 #ifdef FZ_WINDOWS
 		LeaveCriticalSection(m_);
 #else
@@ -136,6 +213,8 @@ public:
 #endif
 	}
 
+	explicit operator bool() const { return locked_; }
+
 private:
 	friend class condition;
 
