@@ -208,14 +208,14 @@ public:
 	typedef int socket_t;
 #endif
 
+	int close();
+
 protected:
 	friend class socket_thread;
 
 	socket_base(thread_pool& pool, event_handler* evt_handler, socket_event_source* ev_source);
 	virtual ~socket_base() = default;
 
-	int close();
-
 	// Note: Unlocks the lock.
 	void detach_thread(scoped_lock & l);
 
@@ -271,6 +271,20 @@ public:
 
 	explicit operator bool() const { return fd_ != -1; }
 
+	/**
+	 * \brief Returns remote address of a connected socket
+	 *
+	 * \return empty string on error
+	 */
+	std::string peer_ip(bool strip_zone_index = false) const;
+
+	/**
+	 * \brief Returns remote port of a connected socket
+	 *
+	 * \return -1 on error
+	 */
+	int peer_port(int& error) const;
+
 private:
 	socket_base::socket_t fd_{-1};
 };
@@ -293,6 +307,8 @@ public:
 	listen_socket(listen_socket const&) = delete;
 	listen_socket& operator=(listen_socket const&) = delete;
 
+	static std::unique_ptr<listen_socket> from_descriptor(socket_descriptor && desc, thread_pool & pool, int & error, fz::event_handler * handler = nullptr);
+
 	/**
 	 * \brief Starts listening
 	 *
@@ -306,7 +322,7 @@ public:
 	std::unique_ptr<socket> accept(int& error, fz::event_handler * handler = nullptr);
 
 	/**
-	 * \brief  Like accept, but only returns a socket descriptor.
+	 * \brief Like accept, but only returns a socket descriptor.
 	 *
 	 * Best suited for tight accept loops where the descriptor is handed
 	 * off to other threads.
@@ -510,20 +526,20 @@ public:
 	virtual int write(void const* buffer, unsigned int size, int& error) override;
 
 	/**
-	* \brief Returns remote address of a connected socket
-	*
-	* \return empty string on error
-	*/
+	 * \brief Returns remote address of a connected socket
+	 *
+	 * \return empty string on error
+	 */
 	std::string peer_ip(bool strip_zone_index = false) const;
 
 	/// Returns the hostname passed to connect()
 	virtual native_string peer_host() const override;
 
 	/**
-	* \brief Returns remote port of a connected socket
-	*
-	* \return -1 on error
-	*/
+	 * \brief Returns remote port of a connected socket
+	 *
+	 * \return -1 on error
+	 */
 	virtual int peer_port(int& error) const override;
 
 	/**
@@ -813,6 +829,9 @@ private:
 #ifndef EHOSTDOWN
 #define EHOSTDOWN WSAEHOSTDOWN
 #endif
+#ifndef ESOCKTNOSUPPORT
+#define ESOCKTNOSUPPORT WSAESOCKTNOSUPPORT
+#endif
 
 // For the future:
 // Handle ERROR_NETNAME_DELETED=64
